# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..types.project import Project
from ..types.project_label_config import ProjectLabelConfig
from .exports.client import AsyncExportsClient, ExportsClient
from .types.projects_create_response import ProjectsCreateResponse
from .types.projects_import_tasks_response import ProjectsImportTasksResponse
from .types.projects_list_response import ProjectsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.exports = ExportsClient(client_wrapper=self._client_wrapper)

    def list(
        self,
        *,
        ordering: typing.Optional[str] = None,
        ids: typing.Optional[str] = None,
        title: typing.Optional[str] = None,
        page: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Project]:
        """
        Return a list of the projects within your organization.

        To perform most tasks with the Label Studio API, you must specify the project ID, sometimes referred to as the `pk`. The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using this API call.

        To retrieve a list of your Label Studio projects, update the following command to match your own environment.
        Replace the domain name, port, and authorization token, then run the following from the command line:

        ```bash
        curl -X GET https://localhost:8080/api/projects/ -H 'Authorization: Token abc123'
        ```

        Parameters
        ----------
        ordering : typing.Optional[str]
            Which field to use when ordering the results.

        ids : typing.Optional[str]
            ids

        title : typing.Optional[str]
            title

        page : typing.Optional[int]
            A page number within the paginated result set.

        page_size : typing.Optional[int]
            Number of results to return per page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Project]


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.list()
        """
        page = page or 1
        _response = self._client_wrapper.httpx_client.request(
            "api/projects/",
            method="GET",
            params={"ordering": ordering, "ids": ids, "title": title, "page": page, "page_size": page_size},
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            _parsed_response = pydantic_v1.parse_obj_as(ProjectsListResponse, _response.json())  # type: ignore
            _has_next = True
            _get_next = lambda: self.list(
                ordering=ordering,
                ids=ids,
                title=title,
                page=page + 1,
                page_size=page_size,
                request_options=request_options,
            )
            _items = _parsed_response.results
            return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        label_config: typing.Optional[str] = OMIT,
        expert_instruction: typing.Optional[str] = OMIT,
        show_instruction: typing.Optional[bool] = OMIT,
        show_skip_button: typing.Optional[bool] = OMIT,
        enable_empty_annotation: typing.Optional[bool] = OMIT,
        show_annotation_history: typing.Optional[bool] = OMIT,
        reveal_preannotations_interactively: typing.Optional[bool] = OMIT,
        show_collab_predictions: typing.Optional[bool] = OMIT,
        maximum_annotations: typing.Optional[int] = OMIT,
        color: typing.Optional[str] = OMIT,
        control_weights: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectsCreateResponse:
        """
        Create a project and set up the labeling interface. For more information about setting up projects, see the following:

        - [Create and configure projects](https://labelstud.io/guide/setup_project)
        - [Configure labeling interface](https://labelstud.io/guide/setup)
        - [Project settings](https://labelstud.io/guide/project_settings)

        ```bash
        curl -H Content-Type:application/json -H 'Authorization: Token abc123' -X POST 'https://localhost:8080/api/projects'     --data '{"label_config": "<View>[...]</View>"}'
        ```

        Parameters
        ----------
        title : typing.Optional[str]
            Project title

        description : typing.Optional[str]
            Project description

        label_config : typing.Optional[str]
            Label config in XML format

        expert_instruction : typing.Optional[str]
            Labeling instructions to show to the user

        show_instruction : typing.Optional[bool]
            Show labeling instructions

        show_skip_button : typing.Optional[bool]
            Show skip button

        enable_empty_annotation : typing.Optional[bool]
            Allow empty annotations

        show_annotation_history : typing.Optional[bool]
            Show annotation history

        reveal_preannotations_interactively : typing.Optional[bool]
            Reveal preannotations interactively. If set to True, predictions will be shown to the user only after selecting the area of interest

        show_collab_predictions : typing.Optional[bool]
            Show predictions to annotators

        maximum_annotations : typing.Optional[int]
            Maximum annotations per task

        color : typing.Optional[str]
            Project color in HEX format

        control_weights : typing.Optional[typing.Dict[str, typing.Any]]
            Dict of weights for each control tag in metric calculation. Each control tag (e.g. label or choice) will have its own key in control weight dict with weight for each label and overall weight. For example, if a bounding box annotation with a control tag named my_bbox should be included with 0.33 weight in agreement calculation, and the first label Car should be twice as important as Airplane, then you need to specify: {'my_bbox': {'type': 'RectangleLabels', 'labels': {'Car': 1.0, 'Airplane': 0.5}, 'overall': 0.33}

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsCreateResponse


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/projects/",
            method="POST",
            json={
                "title": title,
                "description": description,
                "label_config": label_config,
                "expert_instruction": expert_instruction,
                "show_instruction": show_instruction,
                "show_skip_button": show_skip_button,
                "enable_empty_annotation": enable_empty_annotation,
                "show_annotation_history": show_annotation_history,
                "reveal_preannotations_interactively": reveal_preannotations_interactively,
                "show_collab_predictions": show_collab_predictions,
                "maximum_annotations": maximum_annotations,
                "color": color,
                "control_weights": control_weights,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectsCreateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Project:
        """
        Retrieve information about a specific project by project ID. The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Project
            Project information

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Project, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a project by specified project ID. Deleting a project permanently removes all tasks, annotations, and project data from Label Studio.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(self, id: int, *, request: Project, request_options: typing.Optional[RequestOptions] = None) -> Project:
        """
        Update the project settings for a specific project. For more information, see the following:

        - [Create and configure projects](https://labelstud.io/guide/setup_project)
        - [Configure labeling interface](https://labelstud.io/guide/setup)
        - [Project settings](https://labelstud.io/guide/project_settings)

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        <Warning>
        If you are modifying the labeling config for project that has in-progress work, note the following:
        * You cannot remove labels or change the type of labeling being performed unless you delete any existing annotations that are using those labels.
        * If you make changes to the labeling configuration, any tabs that you might have created in the Data Manager are removed.
        </Warning>

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request : Project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Project


        Examples
        --------
        from label_studio_sdk import Project
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.update(
            id=1,
            request=Project(),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Project, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def import_tasks(
        self,
        id: int,
        *,
        request: typing.Sequence[typing.Dict[str, typing.Any]],
        commit_to_project: typing.Optional[bool] = None,
        return_task_ids: typing.Optional[bool] = None,
        preannotated_from_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectsImportTasksResponse:
        """
        Use this API endpoint to import labeling tasks in bulk. Note that each POST request is limited at 250K tasks and 200 MB.
        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).
        
        <Note>
        Imported data is verified against a project *label_config* and must include all variables that were used in the *label_config*.
        
        For example, if the label configuration has a _$text_ variable, then each item in a data object must include a `text` field.
        </Note>
        
        There are three possible ways to import tasks with this endpoint:
        
        #### 1\. **POST with data**
        
        Send JSON tasks as POST data. Only JSON is supported for POSTing files directly.
        
        Update this example to specify your authorization token and Label Studio instance host, then run the following from
        the command line:
        
        ```bash
        curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \
        -X POST 'https://localhost:8080/api/projects/1/import' --data '[{"text": "Some text 1"}, {"text": "Some text 2"}]'
        ```
        
        #### 2\. **POST with files**
        
        Send tasks as files. You can attach multiple files with different names.
        
        - **JSON**: text files in JavaScript object notation format
        - **CSV**: text files with tables in Comma Separated Values format
        - **TSV**: text files with tables in Tab Separated Value format
        - **TXT**: simple text files are similar to CSV with one column and no header, supported for projects with one source only
        
        Update this example to specify your authorization token, Label Studio instance host, and file name and path,
        then run the following from the command line:
        
        ```bash
        curl -H 'Authorization: Token abc123' \
        -X POST 'https://localhost:8080/api/projects/1/import' -F ‘file=@path/to/my_file.csv’
        ```
        
        #### 3\. **POST with URL**
        
        You can also provide a URL to a file with labeling tasks. Supported file formats are the same as in option 2.
        
        ```bash
        curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \
        -X POST 'https://localhost:8080/api/projects/1/import' \
        --data '[{"url": "http://example.com/test1.csv"}, {"url": "http://example.com/test2.csv"}]'
        ```
        
        <br>
        
        Parameters
        ----------
        id : int
            A unique integer value identifying this project.
        
        request : typing.Sequence[typing.Dict[str, typing.Any]]
        
        commit_to_project : typing.Optional[bool]
            Set to "true" to immediately commit tasks to the project.
        
        return_task_ids : typing.Optional[bool]
            Set to "true" to return task IDs in the response.
        
        preannotated_from_fields : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            List of fields to preannotate from the task data. For example, if you provide a list of `{"text": "text", "prediction": "label"}` items in the request, the system will create a task with the `text` field and a prediction with the `label` field when `preannoted_from_fields=["prediction"]`.
        
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.
        
        Returns
        -------
        ProjectsImportTasksResponse
            Tasks successfully imported
        
        Examples
        --------
        from label_studio_sdk.client import LabelStudio
        
        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.import_tasks(
            id=1,
            request=[{}],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/import",
            method="POST",
            params={
                "commit_to_project": commit_to_project,
                "return_task_ids": return_task_ids,
                "preannotated_from_fields": preannotated_from_fields,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectsImportTasksResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic_v1.parse_obj_as(str, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def validate_config(
        self, id: int, *, request: ProjectLabelConfig, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectLabelConfig:
        """
        Determine whether the label configuration for a specific project is valid. For more information about setting up labeling configs, see [Configure labeling interface](https://labelstud.io/guide/setup) and our [Tags reference](https://labelstud.io/tags/).

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request : ProjectLabelConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectLabelConfig


        Examples
        --------
        from label_studio_sdk import ProjectLabelConfig
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.validate_config(
            id=1,
            request=ProjectLabelConfig(
                label_config="label_config",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/validate/",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectLabelConfig, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.exports = AsyncExportsClient(client_wrapper=self._client_wrapper)

    async def list(
        self,
        *,
        ordering: typing.Optional[str] = None,
        ids: typing.Optional[str] = None,
        title: typing.Optional[str] = None,
        page: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Project]:
        """
        Return a list of the projects within your organization.

        To perform most tasks with the Label Studio API, you must specify the project ID, sometimes referred to as the `pk`. The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using this API call.

        To retrieve a list of your Label Studio projects, update the following command to match your own environment.
        Replace the domain name, port, and authorization token, then run the following from the command line:

        ```bash
        curl -X GET https://localhost:8080/api/projects/ -H 'Authorization: Token abc123'
        ```

        Parameters
        ----------
        ordering : typing.Optional[str]
            Which field to use when ordering the results.

        ids : typing.Optional[str]
            ids

        title : typing.Optional[str]
            title

        page : typing.Optional[int]
            A page number within the paginated result set.

        page_size : typing.Optional[int]
            Number of results to return per page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Project]


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.list()
        """
        page = page or 1
        _response = await self._client_wrapper.httpx_client.request(
            "api/projects/",
            method="GET",
            params={"ordering": ordering, "ids": ids, "title": title, "page": page, "page_size": page_size},
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            _parsed_response = pydantic_v1.parse_obj_as(ProjectsListResponse, _response.json())  # type: ignore
            _has_next = True
            _get_next = lambda: self.list(
                ordering=ordering,
                ids=ids,
                title=title,
                page=page + 1,
                page_size=page_size,
                request_options=request_options,
            )
            _items = _parsed_response.results
            return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        label_config: typing.Optional[str] = OMIT,
        expert_instruction: typing.Optional[str] = OMIT,
        show_instruction: typing.Optional[bool] = OMIT,
        show_skip_button: typing.Optional[bool] = OMIT,
        enable_empty_annotation: typing.Optional[bool] = OMIT,
        show_annotation_history: typing.Optional[bool] = OMIT,
        reveal_preannotations_interactively: typing.Optional[bool] = OMIT,
        show_collab_predictions: typing.Optional[bool] = OMIT,
        maximum_annotations: typing.Optional[int] = OMIT,
        color: typing.Optional[str] = OMIT,
        control_weights: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectsCreateResponse:
        """
        Create a project and set up the labeling interface. For more information about setting up projects, see the following:

        - [Create and configure projects](https://labelstud.io/guide/setup_project)
        - [Configure labeling interface](https://labelstud.io/guide/setup)
        - [Project settings](https://labelstud.io/guide/project_settings)

        ```bash
        curl -H Content-Type:application/json -H 'Authorization: Token abc123' -X POST 'https://localhost:8080/api/projects'     --data '{"label_config": "<View>[...]</View>"}'
        ```

        Parameters
        ----------
        title : typing.Optional[str]
            Project title

        description : typing.Optional[str]
            Project description

        label_config : typing.Optional[str]
            Label config in XML format

        expert_instruction : typing.Optional[str]
            Labeling instructions to show to the user

        show_instruction : typing.Optional[bool]
            Show labeling instructions

        show_skip_button : typing.Optional[bool]
            Show skip button

        enable_empty_annotation : typing.Optional[bool]
            Allow empty annotations

        show_annotation_history : typing.Optional[bool]
            Show annotation history

        reveal_preannotations_interactively : typing.Optional[bool]
            Reveal preannotations interactively. If set to True, predictions will be shown to the user only after selecting the area of interest

        show_collab_predictions : typing.Optional[bool]
            Show predictions to annotators

        maximum_annotations : typing.Optional[int]
            Maximum annotations per task

        color : typing.Optional[str]
            Project color in HEX format

        control_weights : typing.Optional[typing.Dict[str, typing.Any]]
            Dict of weights for each control tag in metric calculation. Each control tag (e.g. label or choice) will have its own key in control weight dict with weight for each label and overall weight. For example, if a bounding box annotation with a control tag named my_bbox should be included with 0.33 weight in agreement calculation, and the first label Car should be twice as important as Airplane, then you need to specify: {'my_bbox': {'type': 'RectangleLabels', 'labels': {'Car': 1.0, 'Airplane': 0.5}, 'overall': 0.33}

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsCreateResponse


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.create()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/projects/",
            method="POST",
            json={
                "title": title,
                "description": description,
                "label_config": label_config,
                "expert_instruction": expert_instruction,
                "show_instruction": show_instruction,
                "show_skip_button": show_skip_button,
                "enable_empty_annotation": enable_empty_annotation,
                "show_annotation_history": show_annotation_history,
                "reveal_preannotations_interactively": reveal_preannotations_interactively,
                "show_collab_predictions": show_collab_predictions,
                "maximum_annotations": maximum_annotations,
                "color": color,
                "control_weights": control_weights,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectsCreateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Project:
        """
        Retrieve information about a specific project by project ID. The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Project
            Project information

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.get(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Project, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a project by specified project ID. Deleting a project permanently removes all tasks, annotations, and project data from Label Studio.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.delete(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, id: int, *, request: Project, request_options: typing.Optional[RequestOptions] = None
    ) -> Project:
        """
        Update the project settings for a specific project. For more information, see the following:

        - [Create and configure projects](https://labelstud.io/guide/setup_project)
        - [Configure labeling interface](https://labelstud.io/guide/setup)
        - [Project settings](https://labelstud.io/guide/project_settings)

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        <Warning>
        If you are modifying the labeling config for project that has in-progress work, note the following:
        * You cannot remove labels or change the type of labeling being performed unless you delete any existing annotations that are using those labels.
        * If you make changes to the labeling configuration, any tabs that you might have created in the Data Manager are removed.
        </Warning>

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request : Project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Project


        Examples
        --------
        from label_studio_sdk import Project
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.update(
            id=1,
            request=Project(),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Project, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def import_tasks(
        self,
        id: int,
        *,
        request: typing.Sequence[typing.Dict[str, typing.Any]],
        commit_to_project: typing.Optional[bool] = None,
        return_task_ids: typing.Optional[bool] = None,
        preannotated_from_fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectsImportTasksResponse:
        """
        Use this API endpoint to import labeling tasks in bulk. Note that each POST request is limited at 250K tasks and 200 MB.
        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).
        
        <Note>
        Imported data is verified against a project *label_config* and must include all variables that were used in the *label_config*.
        
        For example, if the label configuration has a _$text_ variable, then each item in a data object must include a `text` field.
        </Note>
        
        There are three possible ways to import tasks with this endpoint:
        
        #### 1\. **POST with data**
        
        Send JSON tasks as POST data. Only JSON is supported for POSTing files directly.
        
        Update this example to specify your authorization token and Label Studio instance host, then run the following from
        the command line:
        
        ```bash
        curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \
        -X POST 'https://localhost:8080/api/projects/1/import' --data '[{"text": "Some text 1"}, {"text": "Some text 2"}]'
        ```
        
        #### 2\. **POST with files**
        
        Send tasks as files. You can attach multiple files with different names.
        
        - **JSON**: text files in JavaScript object notation format
        - **CSV**: text files with tables in Comma Separated Values format
        - **TSV**: text files with tables in Tab Separated Value format
        - **TXT**: simple text files are similar to CSV with one column and no header, supported for projects with one source only
        
        Update this example to specify your authorization token, Label Studio instance host, and file name and path,
        then run the following from the command line:
        
        ```bash
        curl -H 'Authorization: Token abc123' \
        -X POST 'https://localhost:8080/api/projects/1/import' -F ‘file=@path/to/my_file.csv’
        ```
        
        #### 3\. **POST with URL**
        
        You can also provide a URL to a file with labeling tasks. Supported file formats are the same as in option 2.
        
        ```bash
        curl -H 'Content-Type: application/json' -H 'Authorization: Token abc123' \
        -X POST 'https://localhost:8080/api/projects/1/import' \
        --data '[{"url": "http://example.com/test1.csv"}, {"url": "http://example.com/test2.csv"}]'
        ```
        
        <br>
        
        Parameters
        ----------
        id : int
            A unique integer value identifying this project.
        
        request : typing.Sequence[typing.Dict[str, typing.Any]]
        
        commit_to_project : typing.Optional[bool]
            Set to "true" to immediately commit tasks to the project.
        
        return_task_ids : typing.Optional[bool]
            Set to "true" to return task IDs in the response.
        
        preannotated_from_fields : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            List of fields to preannotate from the task data. For example, if you provide a list of `{"text": "text", "prediction": "label"}` items in the request, the system will create a task with the `text` field and a prediction with the `label` field when `preannoted_from_fields=["prediction"]`.
        
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.
        
        Returns
        -------
        ProjectsImportTasksResponse
            Tasks successfully imported
        
        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio
        
        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.import_tasks(
            id=1,
            request=[{}],
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/import",
            method="POST",
            params={
                "commit_to_project": commit_to_project,
                "return_task_ids": return_task_ids,
                "preannotated_from_fields": preannotated_from_fields,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectsImportTasksResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic_v1.parse_obj_as(str, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def validate_config(
        self, id: int, *, request: ProjectLabelConfig, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectLabelConfig:
        """
        Determine whether the label configuration for a specific project is valid. For more information about setting up labeling configs, see [Configure labeling interface](https://labelstud.io/guide/setup) and our [Tags reference](https://labelstud.io/tags/).

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](list).

        Parameters
        ----------
        id : int
            A unique integer value identifying this project.

        request : ProjectLabelConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectLabelConfig


        Examples
        --------
        from label_studio_sdk import ProjectLabelConfig
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.projects.validate_config(
            id=1,
            request=ProjectLabelConfig(
                label_config="label_config",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(id)}/validate/",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectLabelConfig, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ViewsCreateRequestDataFiltersItemsItemFilter = typing.Union[
    typing.Literal[
        "filter:tasks:agreement",
        "filter:tasks:annotations_results",
        "filter:tasks:annotators",
        "filter:tasks:cancelled_annotations",
        "filter:tasks:comments",
        "filter:tasks:completed_at",
        "filter:tasks:created_at",
        "filter:tasks:file_upload",
        "filter:tasks:ground_truth",
        "filter:tasks:id",
        "filter:tasks:inner_id",
        "filter:tasks:predictions_model_versions",
        "filter:tasks:predictions_results",
        "filter:tasks:predictions_score",
        "filter:tasks:reviewed",
        "filter:tasks:reviewers",
        "filter:tasks:reviews_accepted",
        "filter:tasks:reviews_rejected",
        "filter:tasks:total_annotations",
        "filter:tasks:total_predictions",
        "filter:tasks:unresolved_comment_count",
        "filter:tasks:updated_at",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class TaskFilterOptions(pydantic_v1.BaseModel):
    view: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Apply filters from the view ID (a tab from the Data Manager)
    """

    skipped: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    `only` - include all tasks with skipped annotations<br>`exclude` - exclude all tasks with skipped annotations
    """

    finished: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    `only` - include all finished tasks (is_labeled = true)<br>`exclude` - exclude all finished tasks
    """

    annotated: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    `only` - include all tasks with at least one not skipped annotation<br>`exclude` - exclude all tasks with at least one not skipped annotation
    """

    only_with_annotations: typing.Optional[bool] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



""" .. include::../docs/utils.md
"""

import logging

from lxml import etree
from collections import defaultdict

logger = logging.getLogger(__name__)

_LABEL_TAGS = {"Label", "Choice"}
_NOT_CONTROL_TAGS = {
    "Filter",
}


def parse_config(config_string):
    """Parse a given Label Studio labeling configuration and return a structured version of the configuration.
    Useful for formatting results for predicted annotations and determining the type(s) of ML models that might
    be relevant to the labeling project.

    Parameters
    ----------
    config_string: str
        Label configuration XML as a string

    Returns
    -------
    dict
        structured config with the form:
    ```json
    {
        "<ControlTag>.name": {
            "type": "ControlTag",
            "to_name": ["<ObjectTag1>.name", "<ObjectTag2>.name"],
            "inputs: [
                {"type": "ObjectTag1", "value": "<ObjectTag1>.value"},
                {"type": "ObjectTag2", "value": "<ObjectTag2>.value"}
            ],
            "labels": ["Label1", "Label2", "Label3"] // taken from "alias" if it exists, else "value"
    }
    ```

    """
    """
    :param config_string: 
    
    """
    if not config_string:
        return {}

    def _is_input_tag(tag):
        return tag.attrib.get("name") and tag.attrib.get("value")

    def _is_output_tag(tag):
        return (
            tag.attrib.get("name")
            and tag.attrib.get("toName")
            and tag.tag not in _NOT_CONTROL_TAGS
        )

    def _get_parent_output_tag_name(tag, outputs):
        # Find parental <Choices> tag for nested tags like <Choices><View><View><Choice>...
        parent = tag
        while True:
            parent = parent.getparent()
            if parent is None:
                return
            name = parent.attrib.get("name")
            if name in outputs:
                return name

    xml_tree = etree.fromstring(config_string)

    inputs, outputs, labels = {}, {}, defaultdict(dict)
    for tag in xml_tree.iter():
        if _is_output_tag(tag):
            tag_info = {"type": tag.tag, "to_name": tag.attrib["toName"].split(",")}
            # Grab conditionals if any
            conditionals = {}
            if tag.attrib.get("perRegion") == "true":
                if tag.attrib.get("whenTagName"):
                    conditionals = {"type": "tag", "name": tag.attrib["whenTagName"]}
                elif tag.attrib.get("whenLabelValue"):
                    conditionals = {
                        "type": "label",
                        "name": tag.attrib["whenLabelValue"],
                    }
                elif tag.attrib.get("whenChoiceValue"):
                    conditionals = {
                        "type": "choice",
                        "name": tag.attrib["whenChoiceValue"],
                    }
            if conditionals:
                tag_info["conditionals"] = conditionals
            outputs[tag.attrib["name"]] = tag_info
        elif _is_input_tag(tag):
            inputs[tag.attrib["name"]] = {
                "type": tag.tag,
                "value": tag.attrib["value"].lstrip("$"),
            }
        if tag.tag not in _LABEL_TAGS:
            continue
        parent_name = _get_parent_output_tag_name(tag, outputs)
        if parent_name is not None:
            actual_value = tag.attrib.get("alias") or tag.attrib.get("value")
            if not actual_value:
                logger.debug(
                    'Inspecting tag {tag_name}... found no "value" or "alias" attributes.'.format(
                        tag_name=etree.tostring(tag, encoding="unicode").strip()[:50]
                    )
                )
            else:
                labels[parent_name][actual_value] = dict(tag.attrib)
    for output_tag, tag_info in outputs.items():
        tag_info["inputs"] = []
        for input_tag_name in tag_info["to_name"]:
            if input_tag_name not in inputs:
                logger.warning(
                    f"to_name={input_tag_name} is specified for output tag name={output_tag}, "
                    "but we can't find it among input tags"
                )
                continue
            tag_info["inputs"].append(inputs[input_tag_name])
        tag_info["labels"] = list(labels[output_tag])
        tag_info["labels_attrs"] = labels[output_tag]
    return outputs


def chunk(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i : i + n]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..types.webhook import Webhook
from ..types.webhook_serializer_for_update import WebhookSerializerForUpdate
from .types.webhooks_update_request_actions_item import WebhooksUpdateRequestActionsItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WebhooksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Webhook]:
        """
        List all webhooks set up for your organization.

        Webhooks in Label Studio let you set up integrations that subscribe to certain events that occur inside Label Studio. When an event is triggered, Label Studio sends an HTTP POST request to the configured webhook URL.

        For more information, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks).

        Parameters
        ----------
        project : typing.Optional[str]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Webhook]


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/webhooks/", method="GET", params={"project": project}, request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Webhook], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request: Webhook, request_options: typing.Optional[RequestOptions] = None) -> Webhook:
        """
        Create a webhook.
        Label Studio provides several out-of-the box webhook events, which you can find listed here: [Available Label Studio webhooks](https://labelstud.io/guide/webhooks#Available-Label-Studio-webhooks).

        If you want to create your own custom webhook, refer to [Create custom events for webhooks in Label Studio](https://labelstud.io/guide/webhook_create).

        <Note>Label Studio makes two main types of events available to integrate with webhooks: project-level task events and organization events. If you want to use organization-level webhook events, you will need to set `LABEL_STUDIO_ALLOW_ORGANIZATION_WEBHOOKS=true`. </Note>

        Parameters
        ----------
        request : Webhook

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Webhook


        Examples
        --------
        from label_studio_sdk import Webhook
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.create(
            request=Webhook(
                url="url",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/webhooks/", method="POST", json=request, request_options=request_options, omit=OMIT
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Webhook, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def info(
        self,
        *,
        organization_only: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Get descriptions of all available webhook actions to set up webhooks. For more information, see the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        organization_only : typing.Optional[bool]
            organization-only or not

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.info()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/webhooks/info/",
            method="GET",
            params={"organization-only": organization_only},
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Webhook:
        """
        Get information about a specific webhook. You will need to provide the webhook ID. You can get this from [List all webhooks](list).

        For more information about webhooks, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks) and the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        id : int
            A unique integer value identifying this webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Webhook


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/webhooks/{jsonable_encoder(id)}/", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Webhook, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a webhook. You will need to provide the webhook ID. You can get this from [List all webhooks](list).

        For more information about webhooks, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks) and the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        id : int
            A unique integer value identifying this webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/webhooks/{jsonable_encoder(id)}/", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        url: str,
        request: WebhookSerializerForUpdate,
        send_payload: typing.Optional[bool] = None,
        send_for_all_actions: typing.Optional[bool] = None,
        headers: typing.Optional[str] = None,
        is_active: typing.Optional[bool] = None,
        actions: typing.Optional[
            typing.Union[WebhooksUpdateRequestActionsItem, typing.Sequence[WebhooksUpdateRequestActionsItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSerializerForUpdate:
        """
        Update a webhook. You will need to provide the webhook ID. You can get this from [List all webhooks](list).

        For more information about webhooks, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks) and the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        id : int
            A unique integer value identifying this webhook.

        url : str
            URL of webhook

        request : WebhookSerializerForUpdate

        send_payload : typing.Optional[bool]
            If value is False send only action

        send_for_all_actions : typing.Optional[bool]
            If value is False - used only for actions from WebhookAction

        headers : typing.Optional[str]
            Key Value Json of headers

        is_active : typing.Optional[bool]
            If value is False the webhook is disabled

        actions : typing.Optional[typing.Union[WebhooksUpdateRequestActionsItem, typing.Sequence[WebhooksUpdateRequestActionsItem]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSerializerForUpdate


        Examples
        --------
        from label_studio_sdk import WebhookSerializerForUpdate
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.update(
            id=1,
            url="url",
            request=WebhookSerializerForUpdate(
                url="url",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/webhooks/{jsonable_encoder(id)}/",
            method="PATCH",
            params={
                "url": url,
                "send_payload": send_payload,
                "send_for_all_actions": send_for_all_actions,
                "headers": headers,
                "is_active": is_active,
                "actions": actions,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(WebhookSerializerForUpdate, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncWebhooksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Webhook]:
        """
        List all webhooks set up for your organization.

        Webhooks in Label Studio let you set up integrations that subscribe to certain events that occur inside Label Studio. When an event is triggered, Label Studio sends an HTTP POST request to the configured webhook URL.

        For more information, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks).

        Parameters
        ----------
        project : typing.Optional[str]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Webhook]


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.webhooks.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/webhooks/", method="GET", params={"project": project}, request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Webhook], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request: Webhook, request_options: typing.Optional[RequestOptions] = None) -> Webhook:
        """
        Create a webhook.
        Label Studio provides several out-of-the box webhook events, which you can find listed here: [Available Label Studio webhooks](https://labelstud.io/guide/webhooks#Available-Label-Studio-webhooks).

        If you want to create your own custom webhook, refer to [Create custom events for webhooks in Label Studio](https://labelstud.io/guide/webhook_create).

        <Note>Label Studio makes two main types of events available to integrate with webhooks: project-level task events and organization events. If you want to use organization-level webhook events, you will need to set `LABEL_STUDIO_ALLOW_ORGANIZATION_WEBHOOKS=true`. </Note>

        Parameters
        ----------
        request : Webhook

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Webhook


        Examples
        --------
        from label_studio_sdk import Webhook
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.webhooks.create(
            request=Webhook(
                url="url",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/webhooks/", method="POST", json=request, request_options=request_options, omit=OMIT
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Webhook, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def info(
        self,
        *,
        organization_only: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Get descriptions of all available webhook actions to set up webhooks. For more information, see the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        organization_only : typing.Optional[bool]
            organization-only or not

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.webhooks.info()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/webhooks/info/",
            method="GET",
            params={"organization-only": organization_only},
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Webhook:
        """
        Get information about a specific webhook. You will need to provide the webhook ID. You can get this from [List all webhooks](list).

        For more information about webhooks, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks) and the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        id : int
            A unique integer value identifying this webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Webhook


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.webhooks.get(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/webhooks/{jsonable_encoder(id)}/", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Webhook, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a webhook. You will need to provide the webhook ID. You can get this from [List all webhooks](list).

        For more information about webhooks, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks) and the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        id : int
            A unique integer value identifying this webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.webhooks.delete(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/webhooks/{jsonable_encoder(id)}/", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        url: str,
        request: WebhookSerializerForUpdate,
        send_payload: typing.Optional[bool] = None,
        send_for_all_actions: typing.Optional[bool] = None,
        headers: typing.Optional[str] = None,
        is_active: typing.Optional[bool] = None,
        actions: typing.Optional[
            typing.Union[WebhooksUpdateRequestActionsItem, typing.Sequence[WebhooksUpdateRequestActionsItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookSerializerForUpdate:
        """
        Update a webhook. You will need to provide the webhook ID. You can get this from [List all webhooks](list).

        For more information about webhooks, see [Set up webhooks in Label Studio](https://labelstud.io/guide/webhooks) and the [Webhook event reference](https://labelstud.io/guide/webhook_reference).

        Parameters
        ----------
        id : int
            A unique integer value identifying this webhook.

        url : str
            URL of webhook

        request : WebhookSerializerForUpdate

        send_payload : typing.Optional[bool]
            If value is False send only action

        send_for_all_actions : typing.Optional[bool]
            If value is False - used only for actions from WebhookAction

        headers : typing.Optional[str]
            Key Value Json of headers

        is_active : typing.Optional[bool]
            If value is False the webhook is disabled

        actions : typing.Optional[typing.Union[WebhooksUpdateRequestActionsItem, typing.Sequence[WebhooksUpdateRequestActionsItem]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookSerializerForUpdate


        Examples
        --------
        from label_studio_sdk import WebhookSerializerForUpdate
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.webhooks.update(
            id=1,
            url="url",
            request=WebhookSerializerForUpdate(
                url="url",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/webhooks/{jsonable_encoder(id)}/",
            method="PATCH",
            params={
                "url": url,
                "send_payload": send_payload,
                "send_for_all_actions": send_for_all_actions,
                "headers": headers,
                "is_active": is_active,
                "actions": actions,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(WebhookSerializerForUpdate, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class S3CreateResponse(pydantic_v1.BaseModel):
    can_delete_objects: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Deletion from storage enabled.
    """

    title: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Project ID
    """

    bucket: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    S3 bucket name
    """

    prefix: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    S3 bucket prefix
    """

    aws_access_key_id: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    AWS_ACCESS_KEY_ID
    """

    aws_secret_access_key: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    AWS_SECRET_ACCESS_KEY
    """

    aws_session_token: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    AWS_SESSION_TOKEN
    """

    aws_sse_kms_key_id: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    AWS SSE KMS Key ID
    """

    region_name: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    AWS Region
    """

    s3endpoint: typing.Optional[str] = pydantic_v1.Field(alias="s3_endpoint", default=None)
    """
    S3 Endpoint
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .actions_create_request_filters import ActionsCreateRequestFilters
from .actions_create_request_filters_conjunction import ActionsCreateRequestFiltersConjunction
from .actions_create_request_filters_items_item import ActionsCreateRequestFiltersItemsItem
from .actions_create_request_filters_items_item_filter import ActionsCreateRequestFiltersItemsItemFilter
from .actions_create_request_filters_items_item_operator import ActionsCreateRequestFiltersItemsItemOperator
from .actions_create_request_id import ActionsCreateRequestId
from .actions_create_request_ordering_item import ActionsCreateRequestOrderingItem
from .actions_create_request_selected_items import ActionsCreateRequestSelectedItems
from .actions_create_request_selected_items_excluded import ActionsCreateRequestSelectedItemsExcluded
from .actions_create_request_selected_items_included import ActionsCreateRequestSelectedItemsIncluded

__all__ = [
    "ActionsCreateRequestFilters",
    "ActionsCreateRequestFiltersConjunction",
    "ActionsCreateRequestFiltersItemsItem",
    "ActionsCreateRequestFiltersItemsItemFilter",
    "ActionsCreateRequestFiltersItemsItemOperator",
    "ActionsCreateRequestId",
    "ActionsCreateRequestOrderingItem",
    "ActionsCreateRequestSelectedItems",
    "ActionsCreateRequestSelectedItemsExcluded",
    "ActionsCreateRequestSelectedItemsIncluded",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ViewsCreateRequestData,
    ViewsCreateRequestDataFilters,
    ViewsCreateRequestDataFiltersConjunction,
    ViewsCreateRequestDataFiltersItemsItem,
    ViewsCreateRequestDataFiltersItemsItemFilter,
    ViewsCreateRequestDataFiltersItemsItemOperator,
    ViewsCreateRequestDataOrderingItem,
    ViewsUpdateRequestData,
    ViewsUpdateRequestDataFilters,
    ViewsUpdateRequestDataFiltersConjunction,
    ViewsUpdateRequestDataFiltersItemsItem,
    ViewsUpdateRequestDataFiltersItemsItemFilter,
    ViewsUpdateRequestDataFiltersItemsItemOperator,
    ViewsUpdateRequestDataOrderingItem,
)

__all__ = [
    "ViewsCreateRequestData",
    "ViewsCreateRequestDataFilters",
    "ViewsCreateRequestDataFiltersConjunction",
    "ViewsCreateRequestDataFiltersItemsItem",
    "ViewsCreateRequestDataFiltersItemsItemFilter",
    "ViewsCreateRequestDataFiltersItemsItemOperator",
    "ViewsCreateRequestDataOrderingItem",
    "ViewsUpdateRequestData",
    "ViewsUpdateRequestDataFilters",
    "ViewsUpdateRequestDataFiltersConjunction",
    "ViewsUpdateRequestDataFiltersItemsItem",
    "ViewsUpdateRequestDataFiltersItemsItemFilter",
    "ViewsUpdateRequestDataFiltersItemsItemOperator",
    "ViewsUpdateRequestDataOrderingItem",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .ml_backend_auth_method import MlBackendAuthMethod
from .ml_backend_state import MlBackendState


class MlBackend(pydantic_v1.BaseModel):
    id: typing.Optional[int] = None
    state: typing.Optional[MlBackendState] = None
    readable_state: typing.Optional[str] = None
    is_interactive: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Used to interactively annotate tasks. If true, model returns one list with results
    """

    url: str = pydantic_v1.Field()
    """
    URL for the machine learning model server
    """

    error_message: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Error message in error state
    """

    title: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Name of the machine learning backend
    """

    auth_method: typing.Optional[MlBackendAuthMethod] = None
    basic_auth_user: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    HTTP Basic Auth user
    """

    basic_auth_pass: typing.Optional[str] = None
    basic_auth_pass_is_set: typing.Optional[str] = None
    description: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Description for the machine learning backend
    """

    extra_params: typing.Optional[typing.Dict[str, typing.Any]] = pydantic_v1.Field(default=None)
    """
    Any extra parameters passed to the ML Backend during the setup
    """

    model_version: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Current model version associated with this machine learning backend
    """

    timeout: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Response model timeout
    """

    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None
    auto_update: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    If false, model version is set by the user, if true - getting latest version from backend.
    """

    project: int

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



import os
import io
import logging
import json


from .utils import get_audio_duration, ensure_dir, download, get_annotator


logger = logging.getLogger(__name__)


def convert_to_asr_json_manifest(
    input_data, output_dir, data_key, project_dir, upload_dir, download_resources
):
    audio_dir_rel = "audio"
    output_audio_dir = os.path.join(output_dir, audio_dir_rel)
    ensure_dir(output_dir), ensure_dir(output_audio_dir)
    output_file = os.path.join(output_dir, "manifest.json")
    with io.open(output_file, mode="w") as fout:
        for item in input_data:
            audio_path = item["input"][data_key]
            try:
                audio_path = download(
                    audio_path,
                    output_audio_dir,
                    project_dir=project_dir,
                    upload_dir=upload_dir,
                    return_relative_path=True,
                    download_resources=download_resources,
                )
                duration = get_audio_duration(
                    os.path.join(output_audio_dir, os.path.basename(audio_path))
                )
            except:
                logger.info(
                    "Unable to download {image_path} or get audio duration. The item {item} will be skipped".format(
                        image_path=audio_path, item=item
                    ),
                    exc_info=True,
                )
                continue

            for texts in iter(item["output"].values()):
                if len(texts) > 0 and "text" in texts[0]:
                    break

            transcript = texts[0]["text"][0]
            metadata = {
                "audio_filepath": audio_path,
                "duration": duration,
                "text": transcript,
                "annotator": get_annotator(item, default=""),
            }
            json.dump(metadata, fout)
            fout.write("\n")



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import pydantic_v1
from ...core.request_options import RequestOptions
from ...types.local_files_export_storage import LocalFilesExportStorage
from .types.local_create_response import LocalCreateResponse
from .types.local_update_response import LocalUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LocalClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[LocalFilesExportStorage]:
        """
        You can connect a local file directory to Label Studio as a source storage or target storage. Use this API request to get a list of all local file export (target) storage connections for a specific project.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[LocalFilesExportStorage]


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/localfiles", method="GET", params={"project": project}, request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[LocalFilesExportStorage], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        regex_filter: typing.Optional[str] = OMIT,
        use_blob_urls: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LocalCreateResponse:
        """
        Create a new target storage connection to a local file directory.

        For information about the required fields and prerequisites, see [Local storage](https://labelstud.io/guide/storage#Local-storage) in the Label Studio documentation.

        <Tip>After you add the storage, you should validate the connection before attempting to sync your data. Your data will not be exported until you [sync your connection](sync).</Tip>

        Parameters
        ----------
        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Path to local directory

        regex_filter : typing.Optional[str]
            Regex for filtering objects

        use_blob_urls : typing.Optional[bool]
            Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalCreateResponse


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/localfiles",
            method="POST",
            json={
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "regex_filter": regex_filter,
                "use_blob_urls": use_blob_urls,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalCreateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def validate(
        self,
        *,
        id: typing.Optional[int] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        regex_filter: typing.Optional[str] = OMIT,
        use_blob_urls: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Validate a specific local file export storage connection. This is useful to ensure that the storage configuration settings are correct and operational before attempting to export data.

        Parameters
        ----------
        id : typing.Optional[int]
            Storage ID. If set, storage with specified ID will be updated

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Path to local directory

        regex_filter : typing.Optional[str]
            Regex for filtering objects

        use_blob_urls : typing.Optional[bool]
            Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.validate()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/localfiles/validate",
            method="POST",
            json={
                "id": id,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "regex_filter": regex_filter,
                "use_blob_urls": use_blob_urls,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> LocalFilesExportStorage:
        """
        Get a specific local file export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this local files export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalFilesExportStorage


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalFilesExportStorage, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific local file export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Deleting an export/target storage connection does not affect tasks with synced data in Label Studio. If you want to remove the tasks that were synced from the external storage, you will need to delete them manually from within the Label Studio UI or use the [Delete tasks](../../tasks/delete-all-tasks) API.

        Parameters
        ----------
        id : int
            A unique integer value identifying this local files export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        regex_filter: typing.Optional[str] = OMIT,
        use_blob_urls: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LocalUpdateResponse:
        """
        Update a specific local file export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this local files export storage.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Path to local directory

        regex_filter : typing.Optional[str]
            Regex for filtering objects

        use_blob_urls : typing.Optional[bool]
            Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalUpdateResponse


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.update(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "regex_filter": regex_filter,
                "use_blob_urls": use_blob_urls,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalUpdateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def sync(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> LocalFilesExportStorage:
        """
        Sync tasks to an local file export/target storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Sync operations with external local file directories only go one way. They either create tasks from objects in the directory (source/import storage) or push annotations to the output directory (export/target storage). Changing something on the local file side doesn’t guarantee consistency in results.

        <Note>Before proceeding, you should review [How sync operations work - Source storage](https://labelstud.io/guide/storage#Source-storage) to ensure that your data remains secure and private.</Note>

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalFilesExportStorage


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.local.sync(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}/sync",
            method="POST",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalFilesExportStorage, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncLocalClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[LocalFilesExportStorage]:
        """
        You can connect a local file directory to Label Studio as a source storage or target storage. Use this API request to get a list of all local file export (target) storage connections for a specific project.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[LocalFilesExportStorage]


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/localfiles", method="GET", params={"project": project}, request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[LocalFilesExportStorage], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        regex_filter: typing.Optional[str] = OMIT,
        use_blob_urls: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LocalCreateResponse:
        """
        Create a new target storage connection to a local file directory.

        For information about the required fields and prerequisites, see [Local storage](https://labelstud.io/guide/storage#Local-storage) in the Label Studio documentation.

        <Tip>After you add the storage, you should validate the connection before attempting to sync your data. Your data will not be exported until you [sync your connection](sync).</Tip>

        Parameters
        ----------
        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Path to local directory

        regex_filter : typing.Optional[str]
            Regex for filtering objects

        use_blob_urls : typing.Optional[bool]
            Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalCreateResponse


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.create()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/localfiles",
            method="POST",
            json={
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "regex_filter": regex_filter,
                "use_blob_urls": use_blob_urls,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalCreateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def validate(
        self,
        *,
        id: typing.Optional[int] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        regex_filter: typing.Optional[str] = OMIT,
        use_blob_urls: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Validate a specific local file export storage connection. This is useful to ensure that the storage configuration settings are correct and operational before attempting to export data.

        Parameters
        ----------
        id : typing.Optional[int]
            Storage ID. If set, storage with specified ID will be updated

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Path to local directory

        regex_filter : typing.Optional[str]
            Regex for filtering objects

        use_blob_urls : typing.Optional[bool]
            Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.validate()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/localfiles/validate",
            method="POST",
            json={
                "id": id,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "regex_filter": regex_filter,
                "use_blob_urls": use_blob_urls,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> LocalFilesExportStorage:
        """
        Get a specific local file export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this local files export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalFilesExportStorage


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.get(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalFilesExportStorage, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific local file export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Deleting an export/target storage connection does not affect tasks with synced data in Label Studio. If you want to remove the tasks that were synced from the external storage, you will need to delete them manually from within the Label Studio UI or use the [Delete tasks](../../tasks/delete-all-tasks) API.

        Parameters
        ----------
        id : int
            A unique integer value identifying this local files export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.delete(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        regex_filter: typing.Optional[str] = OMIT,
        use_blob_urls: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LocalUpdateResponse:
        """
        Update a specific local file export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this local files export storage.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Path to local directory

        regex_filter : typing.Optional[str]
            Regex for filtering objects

        use_blob_urls : typing.Optional[bool]
            Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalUpdateResponse


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.update(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "regex_filter": regex_filter,
                "use_blob_urls": use_blob_urls,
            },
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalUpdateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def sync(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> LocalFilesExportStorage:
        """
        Sync tasks to an local file export/target storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Sync operations with external local file directories only go one way. They either create tasks from objects in the directory (source/import storage) or push annotations to the output directory (export/target storage). Changing something on the local file side doesn’t guarantee consistency in results.

        <Note>Before proceeding, you should review [How sync operations work - Source storage](https://labelstud.io/guide/storage#Source-storage) to ensure that your data remains secure and private.</Note>

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LocalFilesExportStorage


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.export_storage.local.sync(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/localfiles/{jsonable_encoder(id)}/sync",
            method="POST",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LocalFilesExportStorage, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .views_create_request_data import ViewsCreateRequestData
from .views_create_request_data_filters import ViewsCreateRequestDataFilters
from .views_create_request_data_filters_conjunction import ViewsCreateRequestDataFiltersConjunction
from .views_create_request_data_filters_items_item import ViewsCreateRequestDataFiltersItemsItem
from .views_create_request_data_filters_items_item_filter import ViewsCreateRequestDataFiltersItemsItemFilter
from .views_create_request_data_filters_items_item_operator import ViewsCreateRequestDataFiltersItemsItemOperator
from .views_create_request_data_ordering_item import ViewsCreateRequestDataOrderingItem
from .views_update_request_data import ViewsUpdateRequestData
from .views_update_request_data_filters import ViewsUpdateRequestDataFilters
from .views_update_request_data_filters_conjunction import ViewsUpdateRequestDataFiltersConjunction
from .views_update_request_data_filters_items_item import ViewsUpdateRequestDataFiltersItemsItem
from .views_update_request_data_filters_items_item_filter import ViewsUpdateRequestDataFiltersItemsItemFilter
from .views_update_request_data_filters_items_item_operator import ViewsUpdateRequestDataFiltersItemsItemOperator
from .views_update_request_data_ordering_item import ViewsUpdateRequestDataOrderingItem

__all__ = [
    "ViewsCreateRequestData",
    "ViewsCreateRequestDataFilters",
    "ViewsCreateRequestDataFiltersConjunction",
    "ViewsCreateRequestDataFiltersItemsItem",
    "ViewsCreateRequestDataFiltersItemsItemFilter",
    "ViewsCreateRequestDataFiltersItemsItemOperator",
    "ViewsCreateRequestDataOrderingItem",
    "ViewsUpdateRequestData",
    "ViewsUpdateRequestDataFilters",
    "ViewsUpdateRequestDataFiltersConjunction",
    "ViewsUpdateRequestDataFiltersItemsItem",
    "ViewsUpdateRequestDataFiltersItemsItemFilter",
    "ViewsUpdateRequestDataFiltersItemsItemOperator",
    "ViewsUpdateRequestDataOrderingItem",
]



import apache_beam as beam
from registry import dataloader, call_dataloader


class CallDataLoader(beam.DoFn):
    def __init__(self, dataloader_name):
        self.dataloader_name = dataloader_name

    def setup(self):
        print("Setup!")
        pass

    def process(self, element, *args, **kwargs):
        return call_dataloader(self.dataloader_name, element, *args, **kwargs)

    def teardown(self):
        print("Teardown!")
        pass


@dataloader("test")
def f(i, k):
    print("I!!", k)
    yield i


with beam.Pipeline() as pipeline:
    results = (
        pipeline
        | "Get storage keys" >> beam.io.gcp.gcsio.list_files("bucket")
        | "Run dataloader"
        >> beam.FlatMap(fn=CallDataLoader("test"), **dataloader_kwargs)
        | "Load into Vector DB" >> beam.ParDo(fn=LoadDataRecordInDB)
    )



# This file was auto-generated by Fern from our API Definition.

from .azure_create_response import AzureCreateResponse
from .azure_update_response import AzureUpdateResponse

__all__ = ["AzureCreateResponse", "AzureUpdateResponse"]



"""
Original RLE JS code from https://github.com/thi-ng/umbrella/blob/develop/packages/rle-pack/src/index.ts

export const decode = (src: Uint8Array) => {
    const input = new BitInputStream(src);
    const num = input.read(32);
    const wordSize = input.read(5) + 1;
    const rleSizes = [0, 0, 0, 0].map(() => input.read(4) + 1);
    const out = arrayForWordSize(wordSize, num);
    let x, j;
    for (let i = 0; i < num; ) {
        x = input.readBit();
        j = i + 1 + input.read(rleSizes[input.read(2)]);
        if (x) {
            out.fill(input.read(wordSize), i, j);
            i = j;
        } else {
            for (; i < j; i++) {
                out[i] = input.read(wordSize);
            }
        }
    }
    return out;
};

const arrayForWordSize = (ws: number, n: number) => {
    return new (ws < 9 ? Uint8Array : ws < 17 ? Uint16Array : Uint32Array)(n);
};
"""

import os
import uuid
import numpy as np
import logging

from PIL import Image
from collections import defaultdict
from itertools import groupby

logger = logging.getLogger(__name__)


### Brush Export ###


class InputStream:
    def __init__(self, data):
        self.data = data
        self.i = 0

    def read(self, size):
        out = self.data[self.i : self.i + size]
        self.i += size
        return int(out, 2)


def access_bit(data, num):
    """from bytes array to bits by num position"""
    base = int(num // 8)
    shift = 7 - int(num % 8)
    return (data[base] & (1 << shift)) >> shift


def bytes2bit(data):
    """get bit string from bytes data"""
    return "".join([str(access_bit(data, i)) for i in range(len(data) * 8)])


def decode_rle(rle, print_params: bool = False):
    """from LS RLE to numpy uint8 3d image [width, height, channel]

    Args:
        print_params (bool, optional): If true, a RLE parameters print statement is suppressed
    """
    input = InputStream(bytes2bit(rle))
    num = input.read(32)
    word_size = input.read(5) + 1
    rle_sizes = [input.read(4) + 1 for _ in range(4)]

    if print_params:
        print(
            "RLE params:", num, "values", word_size, "word_size", rle_sizes, "rle_sizes"
        )

    i = 0
    out = np.zeros(num, dtype=np.uint8)
    while i < num:
        x = input.read(1)
        j = i + 1 + input.read(rle_sizes[input.read(2)])
        if x:
            val = input.read(word_size)
            out[i:j] = val
            i = j
        else:
            while i < j:
                val = input.read(word_size)
                out[i] = val
                i += 1
    return out


def decode_from_annotation(from_name, results):
    """from LS annotation to {"tag_name + label_name": [numpy uint8 image (width x height)]}"""
    layers = {}
    counters = defaultdict(int)
    for result in results:
        key = (
            "brushlabels"
            if result["type"].lower() == "brushlabels"
            else ("labels" if result["type"].lower() == "labels" else None)
        )
        if key is None or "rle" not in result:
            continue

        rle = result["rle"]
        width = result["original_width"]
        height = result["original_height"]
        labels = result[key] if key in result else ["no_label"]
        name = from_name + "-" + "-".join(labels)

        # result count
        i = str(counters[name])
        counters[name] += 1
        name += "-" + i

        image = decode_rle(rle)
        layers[name] = np.reshape(image, [height, width, 4])[:, :, 3]
    return layers


def save_brush_images_from_annotation(
    task_id,
    annotation_id,
    completed_by,
    from_name,
    results,
    out_dir,
    out_format="numpy",
):
    layers = decode_from_annotation(from_name, results)
    if isinstance(completed_by, dict):
        email = completed_by.get("email", "")
    else:
        email = str(completed_by)
    email = "".join(
        x for x in email if x.isalnum() or x == "@" or x == "."
    )  # sanitize filename

    for name in layers:
        sanitized_name = name.replace("/", "-").replace("\\", "-")

        filename = os.path.join(
            out_dir,
            "task-"
            + str(task_id)
            + "-annotation-"
            + str(annotation_id)
            + "-by-"
            + email
            + "-"
            + sanitized_name,
        )
        image = layers[name]
        logger.debug(f"Save image to {filename}")
        if out_format == "numpy":
            np.save(filename, image)
        elif out_format == "png":
            im = Image.fromarray(image)
            im.save(filename + ".png")
        else:
            raise Exception("Unknown output format for brush converter")


def convert_task(item, out_dir, out_format="numpy"):
    """Task with multiple annotations to brush images, out_format = numpy | png"""
    for from_name, results in item["output"].items():
        save_brush_images_from_annotation(
            item["id"],
            item["annotation_id"],
            item["completed_by"],
            from_name,
            results,
            out_dir,
            out_format,
        )


def convert_task_dir(items, out_dir, out_format="numpy"):
    """Directory with tasks and annotation to brush images, out_format = numpy | png"""
    for item in items:
        convert_task(item, out_dir, out_format)


# convert_task_dir('/ls/test/completions', '/ls/test/completions/output', 'numpy')


### Brush Import ###


def bits2byte(arr_str, n=8):
    """Convert bits back to byte

    :param arr_str:  string with the bit array
    :type arr_str: str
    :param n: number of bits to separate the arr string into
    :type n: int
    :return rle:
    :type rle: list
    """
    rle = []
    numbers = [arr_str[i : i + n] for i in range(0, len(arr_str), n)]
    for i in numbers:
        rle.append(int(i, 2))
    return rle


# Shamelessly plagiarized from https://stackoverflow.com/a/32681075/6051733
def base_rle_encode(inarray):
    """run length encoding. Partial credit to R rle function.
    Multi datatype arrays catered for including non Numpy
    returns: tuple (runlengths, startpositions, values)"""
    ia = np.asarray(inarray)  # force numpy
    n = len(ia)
    if n == 0:
        return None, None, None
    else:
        y = ia[1:] != ia[:-1]  # pairwise unequal (string safe)
        i = np.append(np.where(y), n - 1)  # must include last element posi
        z = np.diff(np.append(-1, i))  # run lengths
        p = np.cumsum(np.append(0, z))[:-1]  # positions
        return z, p, ia[i]


def encode_rle(arr, wordsize=8, rle_sizes=[3, 4, 8, 16]):
    """Encode a 1d array to rle


    :param arr: flattened np.array from a 4d image (R, G, B, alpha)
    :type arr: np.array
    :param wordsize: wordsize bits for decoding, default is 8
    :type wordsize: int
    :param rle_sizes:  list of ints which state how long a series is of the same number
    :type rle_sizes: list
    :return rle: run length encoded array
    :type rle: list

    """
    # Set length of array in 32 bits
    num = len(arr)
    numbits = f"{num:032b}"

    # put in the wordsize in bits
    wordsizebits = f"{wordsize - 1:05b}"

    # put rle sizes in the bits
    rle_bits = "".join([f"{x - 1:04b}" for x in rle_sizes])

    # combine it into base string
    base_str = numbits + wordsizebits + rle_bits

    # start with creating the rle bite string
    out_str = ""
    for length_reeks, p, value in zip(*base_rle_encode(arr)):
        # TODO: A nice to have but --> this can be optimized but works
        if length_reeks == 1:
            # we state with the first 0 that it has a length of 1
            out_str += "0"
            # We state now the index on the rle sizes
            out_str += "00"

            # the rle size value is 0 for an individual number
            out_str += "000"

            # put the value in a 8 bit string
            out_str += f"{value:08b}"
            state = "single_val"

        elif length_reeks > 1:
            state = "series"
            # rle size = 3
            if length_reeks <= 8:
                # Starting with a 1 indicates that we have started a series
                out_str += "1"

                # index in rle size arr
                out_str += "00"

                # length of array to bits
                out_str += f"{length_reeks - 1:03b}"

                out_str += f"{value:08b}"

            # rle size = 4
            elif 8 < length_reeks <= 16:
                # Starting with a 1 indicates that we have started a series
                out_str += "1"
                out_str += "01"

                # length of array to bits
                out_str += f"{length_reeks - 1:04b}"

                out_str += f"{value:08b}"

            # rle size = 8
            elif 16 < length_reeks <= 256:
                # Starting with a 1 indicates that we have started a series
                out_str += "1"

                out_str += "10"

                # length of array to bits
                out_str += f"{length_reeks - 1:08b}"

                out_str += f"{value:08b}"

            # rle size = 16 or longer
            else:
                length_temp = length_reeks
                while length_temp > 2**16:
                    # Starting with a 1 indicates that we have started a series
                    out_str += "1"

                    out_str += "11"
                    out_str += f"{2 ** 16 - 1:016b}"

                    out_str += f"{value:08b}"
                    length_temp -= 2**16

                # Starting with a 1 indicates that we have started a series
                out_str += "1"

                out_str += "11"
                # length of array to bits
                out_str += f"{length_temp - 1:016b}"

                out_str += f"{value:08b}"

    # make sure that we have an 8 fold lenght otherwise add 0's at the end
    nzfill = 8 - len(base_str + out_str) % 8
    total_str = base_str + out_str
    total_str = total_str + nzfill * "0"

    rle = bits2byte(total_str)

    return rle


def contour2rle(contours, contour_id, img_width, img_height):
    """
    :param contours:  list of contours
    :type contours: list
    :param contour_id: id of contour which you want to translate
    :type contour_id: int
    :param img_width: image shape width
    :type img_width: int
    :param img_height: image shape height
    :type img_height: int
    :return: list of ints in RLE format
    """
    import cv2  # opencv

    mask_im = np.zeros((img_width, img_height, 4))
    mask_contours = cv2.drawContours(
        mask_im, contours, contour_id, color=(0, 255, 0, 100), thickness=-1
    )
    rle_out = encode_rle(mask_contours.ravel().astype(int))
    return rle_out


def mask2rle(mask):
    """Convert mask to RLE

    :param mask: uint8 or int np.array mask with len(shape) == 2 like grayscale image
    :return: list of ints in RLE format
    """
    assert len(mask.shape) == 2, "mask must be 2D np.array"
    assert mask.dtype == np.uint8 or mask.dtype == int, "mask must be uint8 or int"
    array = mask.ravel()
    array = np.repeat(array, 4)  # must be 4 channels
    rle = encode_rle(array)
    return rle


def image2rle(path):
    """Convert mask image (jpg, png) to RLE

    1. Read image as grayscale
    2. Flatten to 1d array
    3. Threshold > 128
    4. Encode

    :param path: path to image with mask (jpg, png), this image will be thresholded with values > 128 to obtain mask,
                 so you can mark background as black and foreground as white
    :return: list of ints in RLE format
    """
    with Image.open(path).convert("L") as image:
        mask = np.array((np.array(image) > 128) * 255, dtype=np.uint8)
        array = mask.ravel()
        array = np.repeat(array, 4)
        rle = encode_rle(array)
        return rle, image.size[0], image.size[1]


def image2annotation(
    path,
    label_name,
    from_name,
    to_name,
    ground_truth=False,
    model_version=None,
    score=None,
):
    """Convert image with mask to brush RLE annotation

    :param path: path to image with mask (jpg, png), this image will be thresholded with values > 128 to obtain mask,
                 so you can mark background as black and foreground as white
    :param label_name: label name from labeling config (<Label>)
    :param from_name: brush tag name (<BrushLabels>)
    :param to_name: image tag name (<Image>)
    :param ground_truth: ground truth annotation true/false
    :param model_version: any string, only for predictions
    :param score: model score as float, only for predictions

    :return: dict with Label Studio Annotation or Prediction (Pre-annotation)
    """
    rle, width, height = image2rle(path)
    result = {
        "result": [
            {
                "id": str(uuid.uuid4())[0:8],
                "type": "brushlabels",
                "value": {"rle": rle, "format": "rle", "brushlabels": [label_name]},
                "origin": "manual",
                "to_name": to_name,
                "from_name": from_name,
                "image_rotation": 0,
                "original_width": width,
                "original_height": height,
            }
        ],
    }

    # prediction
    if model_version:
        result["model_version"] = model_version
        result["score"] = score

    # annotation
    else:
        result["ground_truth"] = ground_truth

    return result



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Annotation,
    AnnotationFilterOptions,
    AnnotationLastAction,
    AzureBlobExportStorage,
    AzureBlobExportStorageStatus,
    AzureBlobImportStorage,
    AzureBlobImportStorageStatus,
    BaseTask,
    BaseUser,
    ConvertedFormat,
    ConvertedFormatStatus,
    Export,
    ExportConvert,
    ExportCreate,
    ExportCreateStatus,
    ExportStatus,
    FileUpload,
    Filter,
    FilterGroup,
    GcsExportStorage,
    GcsExportStorageStatus,
    GcsImportStorage,
    GcsImportStorageStatus,
    LocalFilesExportStorage,
    LocalFilesExportStorageStatus,
    LocalFilesImportStorage,
    LocalFilesImportStorageStatus,
    MlBackend,
    MlBackendAuthMethod,
    MlBackendState,
    Prediction,
    Project,
    ProjectImport,
    ProjectImportStatus,
    ProjectLabelConfig,
    ProjectSampling,
    ProjectSkipQueue,
    RedisExportStorage,
    RedisExportStorageStatus,
    RedisImportStorage,
    RedisImportStorageStatus,
    S3ExportStorage,
    S3ExportStorageStatus,
    S3ImportStorage,
    S3ImportStorageStatus,
    SerializationOption,
    SerializationOptions,
    Task,
    TaskFilterOptions,
    UserSimple,
    View,
    Webhook,
    WebhookActionsItem,
    WebhookSerializerForUpdate,
    WebhookSerializerForUpdateActionsItem,
)
from .errors import BadRequestError, InternalServerError
from . import (
    actions,
    annotations,
    export_storage,
    files,
    import_storage,
    ml,
    predictions,
    projects,
    tasks,
    users,
    views,
    webhooks,
)
from ._legacy import Client
from .actions import (
    ActionsCreateRequestFilters,
    ActionsCreateRequestFiltersConjunction,
    ActionsCreateRequestFiltersItemsItem,
    ActionsCreateRequestFiltersItemsItemFilter,
    ActionsCreateRequestFiltersItemsItemOperator,
    ActionsCreateRequestId,
    ActionsCreateRequestOrderingItem,
    ActionsCreateRequestSelectedItems,
    ActionsCreateRequestSelectedItemsExcluded,
    ActionsCreateRequestSelectedItemsIncluded,
)
from .environment import LabelStudioEnvironment
from .export_storage import ExportStorageListTypesResponseItem
from .import_storage import ImportStorageListTypesResponseItem
from .ml import (
    MlCreateRequestAuthMethod,
    MlCreateResponse,
    MlCreateResponseAuthMethod,
    MlUpdateRequestAuthMethod,
    MlUpdateResponse,
    MlUpdateResponseAuthMethod,
)
from .projects import ProjectsCreateResponse, ProjectsImportTasksResponse, ProjectsListResponse
from .tasks import TasksListRequestFields, TasksListResponse
from .users import UsersGetTokenResponse, UsersResetTokenResponse
from .version import __version__
from .views import (
    ViewsCreateRequestData,
    ViewsCreateRequestDataFilters,
    ViewsCreateRequestDataFiltersConjunction,
    ViewsCreateRequestDataFiltersItemsItem,
    ViewsCreateRequestDataFiltersItemsItemFilter,
    ViewsCreateRequestDataFiltersItemsItemOperator,
    ViewsCreateRequestDataOrderingItem,
    ViewsUpdateRequestData,
    ViewsUpdateRequestDataFilters,
    ViewsUpdateRequestDataFiltersConjunction,
    ViewsUpdateRequestDataFiltersItemsItem,
    ViewsUpdateRequestDataFiltersItemsItemFilter,
    ViewsUpdateRequestDataFiltersItemsItemOperator,
    ViewsUpdateRequestDataOrderingItem,
)
from .webhooks import WebhooksUpdateRequestActionsItem

__all__ = [
    "ActionsCreateRequestFilters",
    "ActionsCreateRequestFiltersConjunction",
    "ActionsCreateRequestFiltersItemsItem",
    "ActionsCreateRequestFiltersItemsItemFilter",
    "ActionsCreateRequestFiltersItemsItemOperator",
    "ActionsCreateRequestId",
    "ActionsCreateRequestOrderingItem",
    "ActionsCreateRequestSelectedItems",
    "ActionsCreateRequestSelectedItemsExcluded",
    "ActionsCreateRequestSelectedItemsIncluded",
    "Annotation",
    "AnnotationFilterOptions",
    "AnnotationLastAction",
    "AzureBlobExportStorage",
    "AzureBlobExportStorageStatus",
    "AzureBlobImportStorage",
    "AzureBlobImportStorageStatus",
    "BadRequestError",
    "BaseTask",
    "BaseUser",
    "Client",
    "ConvertedFormat",
    "ConvertedFormatStatus",
    "Export",
    "ExportConvert",
    "ExportCreate",
    "ExportCreateStatus",
    "ExportStatus",
    "ExportStorageListTypesResponseItem",
    "FileUpload",
    "Filter",
    "FilterGroup",
    "GcsExportStorage",
    "GcsExportStorageStatus",
    "GcsImportStorage",
    "GcsImportStorageStatus",
    "ImportStorageListTypesResponseItem",
    "InternalServerError",
    "LabelStudioEnvironment",
    "LocalFilesExportStorage",
    "LocalFilesExportStorageStatus",
    "LocalFilesImportStorage",
    "LocalFilesImportStorageStatus",
    "MlBackend",
    "MlBackendAuthMethod",
    "MlBackendState",
    "MlCreateRequestAuthMethod",
    "MlCreateResponse",
    "MlCreateResponseAuthMethod",
    "MlUpdateRequestAuthMethod",
    "MlUpdateResponse",
    "MlUpdateResponseAuthMethod",
    "Prediction",
    "Project",
    "ProjectImport",
    "ProjectImportStatus",
    "ProjectLabelConfig",
    "ProjectSampling",
    "ProjectSkipQueue",
    "ProjectsCreateResponse",
    "ProjectsImportTasksResponse",
    "ProjectsListResponse",
    "RedisExportStorage",
    "RedisExportStorageStatus",
    "RedisImportStorage",
    "RedisImportStorageStatus",
    "S3ExportStorage",
    "S3ExportStorageStatus",
    "S3ImportStorage",
    "S3ImportStorageStatus",
    "SerializationOption",
    "SerializationOptions",
    "Task",
    "TaskFilterOptions",
    "TasksListRequestFields",
    "TasksListResponse",
    "UserSimple",
    "UsersGetTokenResponse",
    "UsersResetTokenResponse",
    "View",
    "ViewsCreateRequestData",
    "ViewsCreateRequestDataFilters",
    "ViewsCreateRequestDataFiltersConjunction",
    "ViewsCreateRequestDataFiltersItemsItem",
    "ViewsCreateRequestDataFiltersItemsItemFilter",
    "ViewsCreateRequestDataFiltersItemsItemOperator",
    "ViewsCreateRequestDataOrderingItem",
    "ViewsUpdateRequestData",
    "ViewsUpdateRequestDataFilters",
    "ViewsUpdateRequestDataFiltersConjunction",
    "ViewsUpdateRequestDataFiltersItemsItem",
    "ViewsUpdateRequestDataFiltersItemsItemFilter",
    "ViewsUpdateRequestDataFiltersItemsItemOperator",
    "ViewsUpdateRequestDataOrderingItem",
    "Webhook",
    "WebhookActionsItem",
    "WebhookSerializerForUpdate",
    "WebhookSerializerForUpdateActionsItem",
    "WebhooksUpdateRequestActionsItem",
    "__version__",
    "actions",
    "annotations",
    "export_storage",
    "files",
    "import_storage",
    "ml",
    "predictions",
    "projects",
    "tasks",
    "users",
    "views",
    "webhooks",
]



# This file was auto-generated by Fern from our API Definition.

import typing

WebhooksUpdateRequestActionsItem = typing.Union[
    typing.Literal[
        "PROJECT_CREATED",
        "PROJECT_UPDATED",
        "PROJECT_DELETED",
        "TASKS_CREATED",
        "TASKS_DELETED",
        "ANNOTATION_CREATED",
        "ANNOTATIONS_CREATED",
        "ANNOTATION_UPDATED",
        "ANNOTATIONS_DELETED",
        "LABEL_LINK_CREATED",
        "LABEL_LINK_UPDATED",
        "LABEL_LINK_DELETED",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from .users_get_token_response import UsersGetTokenResponse
from .users_reset_token_response import UsersResetTokenResponse

__all__ = ["UsersGetTokenResponse", "UsersResetTokenResponse"]



from typing import Type, Dict, Optional, List, Tuple, Any, Union
from pydantic import BaseModel, Field, confloat

from label_studio_sdk.label_interface.region import Region


def serialize_regions(result):
    """ """
    res = []
    relations = []
    for r in result:
        if isinstance(r, Region):
            res.append(r._dict())
            if r.has_relations:
                relations.append(r._dict_relations())
        else:
            res.append(r)

    return res + relations


class PredictionValue(BaseModel):
    """ """

    model_version: Optional[Any] = None
    score: Optional[float] = 0.00
    result: Optional[List[Union[Dict[str, Any], Region]]]
    # cluster: Optional[Any] = None
    # neighbors: Optional[Any] = None

    class Config:
        allow_population_by_field_name = True

    def serialize(self):
        """ """
        return {
            "model_version": self.model_version,
            "score": self.score,
            "result": serialize_regions(self.result),
        }


class AnnotationValue(BaseModel):
    """ """
    
    was_cancelled: Optional[bool] = False
    ground_truth: Optional[bool] = False
    lead_time: Optional[float] = 0.0
    result_count: Optional[int] = 0
    completed_by: int

    result: Optional[List[Union[Dict[str, Any], Region]]]
    
    class Config:
        allow_population_by_field_name = True

    def serialize(self):
        """ """
        return {
            "was_cancelled": self.was_cancelled,
            "ground_truth": self.ground_truth,
            "lead_time": self.lead_time,
            "result_count": self.result_count,
            "completed_by": self.completed_by,
            "result": serialize_regions(self.result),
        }


class TaskValue(BaseModel):
    """ """
    
    data: Optional[dict]
    annotations: Optional[List[AnnotationValue]]
    predictions: Optional[List[PredictionValue]]



# This file was auto-generated by Fern from our API Definition.

from .import_storage_list_types_response_item import ImportStorageListTypesResponseItem

__all__ = ["ImportStorageListTypesResponseItem"]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..types.prediction import Prediction

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PredictionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        task: typing.Optional[int] = None,
        project: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Prediction]:
        """
        Get a list of all predictions. You can optionally filter these by task or by project. If you want to filter, you will need the project ID and/or task ID. Both of these can be found in the Label Studio URL when viewing a task, or you can use [List all projects](../projects/list) and [Get tasks list](../tasks/list).

        <Note>The terms "predictions" and pre-annotations" are used interchangeably.</Note>

        Predictions can be [imported directly into Label Studio](https://labelstud.io/guide/predictions) or [generated by a connected ML backend](https://labelstud.io/guide/ml.html#Pre-annotations-predictions).

        To import predictions via the API, see [Create prediction](create).

        Parameters
        ----------
        task : typing.Optional[int]
            Filter predictions by task ID

        project : typing.Optional[int]
            Filter predictions by project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Prediction]
            Predictions list

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.predictions.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/predictions/", method="GET", params={"task": task, "project": project}, request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Prediction], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        task: typing.Optional[int] = OMIT,
        result: typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]] = OMIT,
        score: typing.Optional[float] = OMIT,
        model_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prediction:
        """
        If you have predictions generated for your dataset from a model, either as pre-annotated tasks or pre-labeled tasks, you can import the predictions with your dataset into Label Studio for review and correction.

        To import predicted labels into Label Studio, you must use the [Basic Label Studio JSON format](https://labelstud.io/guide/tasks#Basic-Label-Studio-JSON-format) and set up your tasks with the predictions JSON key. The Label Studio ML backend also outputs tasks in this format.

        #### JSON format for predictions

        Label Studio JSON format for pre-annotations must contain two sections:

        - A data object which references the source of the data that the pre-annotations apply to. This can be a URL to an audio file, a pre-signed cloud storage link to an image, plain text, a reference to a CSV file stored in Label Studio, or something else.
        - A predictions array that contains the pre-annotation results for the different types of labeling. See how to add results to the predictions array.

        For more information, see [the JSON format reference in the Label Studio documentation](https://labelstud.io/guide/predictions#JSON-format-for-pre-annotations)

        Parameters
        ----------
        task : typing.Optional[int]
            Task ID for which the prediction is created

        result : typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]]
            Prediction result in JSON format. Read more about the format in [the Label Studio documentation.](https://labelstud.io/guide/predictions)

        score : typing.Optional[float]
            Prediction score. Can be used in Data Manager to sort task by model confidence. Task with the lowest score will be shown first.

        model_version : typing.Optional[str]
            Model version - tag for predictions that can be used to filter tasks in Data Manager, as well as select specific model version for showing preannotations in the labeling interface

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prediction
            Created prediction

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.predictions.create(
            result=[
                {
                    "original_width": 1920,
                    "original_height": 1080,
                    "image_rotation": 0,
                    "from_name": "bboxes",
                    "to_name": "image",
                    "type": "rectanglelabels",
                    "value": {
                        "x": 20,
                        "y": 30,
                        "width": 50,
                        "height": 60,
                        "rotation": 0,
                        "values": {"rectanglelabels": {"0": "Person"}},
                    },
                }
            ],
            score=0.95,
            model_version="yolo-v8",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/predictions/",
            method="POST",
            json={"task": task, "result": result, "score": score, "model_version": model_version},
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Prediction, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Prediction:
        """
        Get details about a specific prediction by its ID. To find the prediction ID, use [List predictions](list).

        For information about the prediction format, see [the JSON format reference in the Label Studio documentation](https://labelstud.io/guide/predictions#JSON-format-for-pre-annotations).

        Parameters
        ----------
        id : int
            Prediction ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prediction
            Prediction details

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.predictions.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/predictions/{jsonable_encoder(id)}/", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Prediction, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a prediction. To find the prediction ID, use [List predictions](list).

        Parameters
        ----------
        id : int
            Prediction ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.predictions.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/predictions/{jsonable_encoder(id)}/", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        task: typing.Optional[int] = OMIT,
        result: typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]] = OMIT,
        score: typing.Optional[float] = OMIT,
        model_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prediction:
        """
        Update a prediction. To find the prediction ID, use [List predictions](list).

        For information about the prediction format, see [the JSON format reference in the Label Studio documentation](https://labelstud.io/guide/predictions#JSON-format-for-pre-annotations).

        Parameters
        ----------
        id : int
            Prediction ID

        task : typing.Optional[int]
            Task ID for which the prediction is created

        result : typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]]
            Prediction result in JSON format. Read more about the format in [the Label Studio documentation.](https://labelstud.io/guide/predictions)

        score : typing.Optional[float]
            Prediction score. Can be used in Data Manager to sort task by model confidence. Task with the lowest score will be shown first.

        model_version : typing.Optional[str]
            Model version - tag for predictions that can be used to filter tasks in Data Manager, as well as select specific model version for showing preannotations in the labeling interface

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prediction
            Updated prediction

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.predictions.update(
            id=1,
            result=[
                {
                    "original_width": 1920,
                    "original_height": 1080,
                    "image_rotation": 0,
                    "from_name": "bboxes",
                    "to_name": "image",
                    "type": "rectanglelabels",
                    "value": {
                        "x": 20,
                        "y": 30,
                        "width": 50,
                        "height": 60,
                        "rotation": 0,
                        "values": {"rectanglelabels": {"0": "Person"}},
                    },
                }
            ],
            score=0.95,
            model_version="yolo-v8",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/predictions/{jsonable_encoder(id)}/",
            method="PATCH",
            json={"task": task, "result": result, "score": score, "model_version": model_version},
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Prediction, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncPredictionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        task: typing.Optional[int] = None,
        project: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Prediction]:
        """
        Get a list of all predictions. You can optionally filter these by task or by project. If you want to filter, you will need the project ID and/or task ID. Both of these can be found in the Label Studio URL when viewing a task, or you can use [List all projects](../projects/list) and [Get tasks list](../tasks/list).

        <Note>The terms "predictions" and pre-annotations" are used interchangeably.</Note>

        Predictions can be [imported directly into Label Studio](https://labelstud.io/guide/predictions) or [generated by a connected ML backend](https://labelstud.io/guide/ml.html#Pre-annotations-predictions).

        To import predictions via the API, see [Create prediction](create).

        Parameters
        ----------
        task : typing.Optional[int]
            Filter predictions by task ID

        project : typing.Optional[int]
            Filter predictions by project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Prediction]
            Predictions list

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.predictions.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/predictions/", method="GET", params={"task": task, "project": project}, request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[Prediction], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        task: typing.Optional[int] = OMIT,
        result: typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]] = OMIT,
        score: typing.Optional[float] = OMIT,
        model_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prediction:
        """
        If you have predictions generated for your dataset from a model, either as pre-annotated tasks or pre-labeled tasks, you can import the predictions with your dataset into Label Studio for review and correction.

        To import predicted labels into Label Studio, you must use the [Basic Label Studio JSON format](https://labelstud.io/guide/tasks#Basic-Label-Studio-JSON-format) and set up your tasks with the predictions JSON key. The Label Studio ML backend also outputs tasks in this format.

        #### JSON format for predictions

        Label Studio JSON format for pre-annotations must contain two sections:

        - A data object which references the source of the data that the pre-annotations apply to. This can be a URL to an audio file, a pre-signed cloud storage link to an image, plain text, a reference to a CSV file stored in Label Studio, or something else.
        - A predictions array that contains the pre-annotation results for the different types of labeling. See how to add results to the predictions array.

        For more information, see [the JSON format reference in the Label Studio documentation](https://labelstud.io/guide/predictions#JSON-format-for-pre-annotations)

        Parameters
        ----------
        task : typing.Optional[int]
            Task ID for which the prediction is created

        result : typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]]
            Prediction result in JSON format. Read more about the format in [the Label Studio documentation.](https://labelstud.io/guide/predictions)

        score : typing.Optional[float]
            Prediction score. Can be used in Data Manager to sort task by model confidence. Task with the lowest score will be shown first.

        model_version : typing.Optional[str]
            Model version - tag for predictions that can be used to filter tasks in Data Manager, as well as select specific model version for showing preannotations in the labeling interface

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prediction
            Created prediction

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.predictions.create(
            result=[
                {
                    "original_width": 1920,
                    "original_height": 1080,
                    "image_rotation": 0,
                    "from_name": "bboxes",
                    "to_name": "image",
                    "type": "rectanglelabels",
                    "value": {
                        "x": 20,
                        "y": 30,
                        "width": 50,
                        "height": 60,
                        "rotation": 0,
                        "values": {"rectanglelabels": {"0": "Person"}},
                    },
                }
            ],
            score=0.95,
            model_version="yolo-v8",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/predictions/",
            method="POST",
            json={"task": task, "result": result, "score": score, "model_version": model_version},
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Prediction, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Prediction:
        """
        Get details about a specific prediction by its ID. To find the prediction ID, use [List predictions](list).

        For information about the prediction format, see [the JSON format reference in the Label Studio documentation](https://labelstud.io/guide/predictions#JSON-format-for-pre-annotations).

        Parameters
        ----------
        id : int
            Prediction ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prediction
            Prediction details

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.predictions.get(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/predictions/{jsonable_encoder(id)}/", method="GET", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Prediction, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a prediction. To find the prediction ID, use [List predictions](list).

        Parameters
        ----------
        id : int
            Prediction ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.predictions.delete(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/predictions/{jsonable_encoder(id)}/", method="DELETE", request_options=request_options
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        task: typing.Optional[int] = OMIT,
        result: typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]] = OMIT,
        score: typing.Optional[float] = OMIT,
        model_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Prediction:
        """
        Update a prediction. To find the prediction ID, use [List predictions](list).

        For information about the prediction format, see [the JSON format reference in the Label Studio documentation](https://labelstud.io/guide/predictions#JSON-format-for-pre-annotations).

        Parameters
        ----------
        id : int
            Prediction ID

        task : typing.Optional[int]
            Task ID for which the prediction is created

        result : typing.Optional[typing.Sequence[typing.Dict[str, typing.Any]]]
            Prediction result in JSON format. Read more about the format in [the Label Studio documentation.](https://labelstud.io/guide/predictions)

        score : typing.Optional[float]
            Prediction score. Can be used in Data Manager to sort task by model confidence. Task with the lowest score will be shown first.

        model_version : typing.Optional[str]
            Model version - tag for predictions that can be used to filter tasks in Data Manager, as well as select specific model version for showing preannotations in the labeling interface

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Prediction
            Updated prediction

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.predictions.update(
            id=1,
            result=[
                {
                    "original_width": 1920,
                    "original_height": 1080,
                    "image_rotation": 0,
                    "from_name": "bboxes",
                    "to_name": "image",
                    "type": "rectanglelabels",
                    "value": {
                        "x": 20,
                        "y": 30,
                        "width": 50,
                        "height": 60,
                        "rotation": 0,
                        "values": {"rectanglelabels": {"0": "Person"}},
                    },
                }
            ],
            score=0.95,
            model_version="yolo-v8",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/predictions/{jsonable_encoder(id)}/",
            method="PATCH",
            json={"task": task, "result": result, "score": score, "model_version": model_version},
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Prediction, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .projects_create_response import ProjectsCreateResponse
from .projects_import_tasks_response import ProjectsImportTasksResponse
from .projects_list_response import ProjectsListResponse

__all__ = ["ProjectsCreateResponse", "ProjectsImportTasksResponse", "ProjectsListResponse"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .azure_blob_export_storage_status import AzureBlobExportStorageStatus


class AzureBlobExportStorage(pydantic_v1.BaseModel):
    id: typing.Optional[int] = None
    type: typing.Optional[str] = None
    synchronizable: typing.Optional[bool] = None
    container: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Azure blob container
    """

    prefix: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Azure blob prefix name
    """

    regex_filter: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Cloud storage regex for filtering objects
    """

    use_blob_urls: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Interpret objects as BLOBs and generate URLs
    """

    account_name: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Azure Blob account name
    """

    account_key: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Azure Blob account key
    """

    last_sync: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Last sync finished time
    """

    last_sync_count: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Count of tasks synced last time
    """

    last_sync_job: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Last sync job ID
    """

    status: typing.Optional[AzureBlobExportStorageStatus] = None
    traceback: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Traceback report for the last failed sync
    """

    meta: typing.Optional[typing.Dict[str, typing.Any]] = pydantic_v1.Field(default=None)
    """
    Meta and debug information about storage processes
    """

    title: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Cloud storage title
    """

    description: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Cloud storage description
    """

    created_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Creation time
    """

    can_delete_objects: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Deletion from storage enabled
    """

    project: int = pydantic_v1.Field()
    """
    A unique integer value identifying this project.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
